import { openArray } from 'zarr';
import { ZarrImage } from './zarrImage';
import { ZarrImagePyramid } from './zarrImagePyramid';

export async function openImage(zarrConfig: {}, isMultichannel = false): Promise<ZarrImage> {
  const zarrArray = await openArray(zarrConfig);
  return ZarrImage.create(zarrArray, isMultichannel);
}

export async function openImagePyramid(baseUrl: string, numLayers: number, isMultichannel = false): Promise<ZarrImagePyramid> {
  const arrQueue = [];
  for (let i = 0; i < numLayers; i++) {
    const config = { store: baseUrl, path: String(i).padStart(2, '0') };
    arrQueue[i] = openImage(config, isMultichannel);
  }
  const zImages = await Promise.all(arrQueue);
  return new ZarrImagePyramid(zImages);
}
