import { ZarrArray, TypedArray } from 'zarr';

// interface Channel {
//   name: string;
// }
export class ZarrImage {
  public imgArray: ZarrArray;
  public isMultichannel: boolean;

  public get shape(): number[] {
    return this.imgArray.shape;
  }

  public get tileShape(): number[] {
    return this.imgArray.chunks;
  }

  public get tileDataShape(): number[] {
    const s = [];
    for (let i = 0; i < this.shape.length; i++) {
      s[i] = Math.ceil(this.shape[i] / this.imgArray.chunks[i]);
    }
    return s;
  }

  public get numTiles(): number {
    return this.tileDataShape.reduce((a, b) => a * b, 1);
  }

  private constructor(zarrArray: ZarrArray, isMultichannel: boolean) {
    this.imgArray = zarrArray;
    this.isMultichannel = isMultichannel;
  }

  public static async create(zarrArray: ZarrArray, isMultichannel = false): Promise<ZarrImage> {
    return new ZarrImage(zarrArray, isMultichannel);
  }

  public async getTile(x: number, y: number): Promise<TypedArray[] | TypedArray> {
    const tile = await this.imgArray.getRawChunk([0, y, x]);
    if (this.isMultichannel) {
      return this.decodeChannels(tile.data, tile.shape[0]);
    }
    return tile.data;
  }

  private decodeChannels(arr: TypedArray, numChannels: number): TypedArray[] {
    const offset = arr.length / numChannels;
    const tileData = [];
    for (let i = 0; i < numChannels; i += 1) {
      const channelData = arr.subarray(offset * i, offset * i + offset);
      tileData.push(channelData);
    }
    return tileData;
  }
}
